@@ -8,6 +8,12 @@
          case GLSL_TYPE_INT:
             data.i[c] = ~ op[0]->value.i[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = ~ op[0]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = ~ op[0]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -41,6 +47,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = -op[0]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = -op[0]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = -op[0]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -59,6 +71,9 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = fabs(op[0]->value.d[c]);
             break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c] < 0 ? -op[0]->value.i64[c] : op[0]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -77,6 +92,9 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = double((op[0]->value.d[c] > 0.0) - (op[0]->value.d[c] < 0.0));
             break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = (op[0]->value.i64[c] > 0) - (op[0]->value.i64[c] < 0);
+            break;
          default:
             unreachable("invalid type");
          }
@@ -87,10 +105,10 @@
       for (unsigned c = 0; c < op[0]->type->components(); c++) {
          switch (op[0]->type->base_type) {
          case GLSL_TYPE_FLOAT:
-            data.f[c] = op[0]->value.f[c] != 0.0F ? 1.0F / op[0]->value.f[c] : 0.0F;
+            data.f[c] = 1.0F / op[0]->value.f[c];
             break;
          case GLSL_TYPE_DOUBLE:
-            data.d[c] = op[0]->value.d[c] != 0.0 ? 1.0 / op[0]->value.d[c] : 0.0;
+            data.d[c] = 1.0 / op[0]->value.d[c];
             break;
          default:
             unreachable("invalid type");
@@ -431,6 +449,294 @@
       }
       break;
 
+   case ir_unop_bitcast_u642d:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT64:
+            data.d[c] = bitcast_u642d(op[0]->value.u64[c]);
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_bitcast_i642d:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.d[c] = bitcast_i642d(op[0]->value.i64[c]);
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_bitcast_d2u64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_DOUBLE:
+            data.u64[c] = bitcast_d2u64(op[0]->value.d[c]);
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_bitcast_d2i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_DOUBLE:
+            data.i64[c] = bitcast_d2i64(op[0]->value.d[c]);
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i642i:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.i[c] = op[0]->value.i64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u642i:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT64:
+            data.i[c] = op[0]->value.u64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i642u:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.u[c] = op[0]->value.i64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u642u:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT64:
+            data.u[c] = op[0]->value.u64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i642b:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] != 0;
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i642f:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.f[c] = op[0]->value.i64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u642f:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT64:
+            data.f[c] = op[0]->value.u64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i642d:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.d[c] = op[0]->value.i64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u642d:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT64:
+            data.d[c] = op[0]->value.u64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i2i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT:
+            data.i64[c] = op[0]->value.i[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u2i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT:
+            data.i64[c] = op[0]->value.u[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_b2i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_BOOL:
+            data.i64[c] = op[0]->value.b[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_f2i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_FLOAT:
+            data.i64[c] = op[0]->value.f[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_d2i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_DOUBLE:
+            data.i64[c] = op[0]->value.d[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i2u64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT:
+            data.u64[c] = op[0]->value.i[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u2u64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT:
+            data.u64[c] = op[0]->value.u[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_f2u64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_FLOAT:
+            data.u64[c] = op[0]->value.f[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_d2u64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_DOUBLE:
+            data.u64[c] = op[0]->value.d[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_u642i64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_UINT64:
+            data.i64[c] = op[0]->value.u64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
+   case ir_unop_i642u64:
+      for (unsigned c = 0; c < op[0]->type->components(); c++) {
+         switch (op[0]->type->base_type) {
+         case GLSL_TYPE_INT64:
+            data.u64[c] = op[0]->value.i64[c];
+            break;
+         default:
+            unreachable("invalid type");
+         }
+      }
+      break;
+
    case ir_unop_trunc:
       for (unsigned c = 0; c < op[0]->type->components(); c++) {
          switch (op[0]->type->base_type) {
@@ -752,6 +1058,38 @@
       memcpy(&data.u[0], &op[0]->value.d[0], sizeof(double));
       break;
 
+   case ir_unop_pack_sampler_2x32:
+      memcpy(&data.u64[0], &op[0]->value.u[0], sizeof(uint64_t));
+      break;
+
+   case ir_unop_pack_image_2x32:
+      memcpy(&data.u64[0], &op[0]->value.u[0], sizeof(uint64_t));
+      break;
+
+   case ir_unop_unpack_sampler_2x32:
+      memcpy(&data.u[0], &op[0]->value.u64[0], sizeof(uint64_t));
+      break;
+
+   case ir_unop_unpack_image_2x32:
+      memcpy(&data.u[0], &op[0]->value.u64[0], sizeof(uint64_t));
+      break;
+
+   case ir_unop_pack_int_2x32:
+      memcpy(&data.i64[0], &op[0]->value.i[0], sizeof(int64_t));
+      break;
+
+   case ir_unop_pack_uint_2x32:
+      memcpy(&data.u64[0], &op[0]->value.u[0], sizeof(uint64_t));
+      break;
+
+   case ir_unop_unpack_int_2x32:
+      memcpy(&data.i[0], &op[0]->value.i64[0], sizeof(int64_t));
+      break;
+
+   case ir_unop_unpack_uint_2x32:
+      memcpy(&data.u[0], &op[0]->value.u64[0], sizeof(uint64_t));
+      break;
+
    case ir_binop_add:
       assert(op[0]->type == op[1]->type || op0_scalar || op1_scalar);
       for (unsigned c = 0, c0 = 0, c1 = 0;
@@ -771,6 +1109,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = op[0]->value.d[c0] + op[1]->value.d[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] + op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] + op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -796,6 +1140,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = op[0]->value.d[c0] - op[1]->value.d[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] - op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] - op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -823,6 +1173,12 @@
             case GLSL_TYPE_DOUBLE:
                data.d[c] = op[0]->value.d[c0] * op[1]->value.d[c1];
                break;
+            case GLSL_TYPE_UINT64:
+               data.u64[c] = op[0]->value.u64[c0] * op[1]->value.u64[c1];
+               break;
+            case GLSL_TYPE_INT64:
+               data.i64[c] = op[0]->value.i64[c0] * op[1]->value.i64[c1];
+               break;
             default:
                unreachable("invalid type");
             }
@@ -846,7 +1202,7 @@
          for (unsigned j = 0; j < p; j++) {
             for (unsigned i = 0; i < n; i++) {
                for (unsigned k = 0; k < m; k++) {
-                  if (op[0]->type->base_type == GLSL_TYPE_DOUBLE)
+                  if (op[0]->type->is_double())
                      data.d[i+n*j] += op[0]->value.d[i+n*k]*op[1]->value.d[k+m*j];
                   else
                      data.f[i+n*j] += op[0]->value.f[i+n*k]*op[1]->value.f[k+m*j];
@@ -875,6 +1231,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = op[0]->value.d[c0] / op[1]->value.d[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[1]->value.u64[c1] == 0 ? 0 : op[0]->value.u64[c0] / op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[1]->value.i64[c1] == 0 ? 0 : op[0]->value.i64[c0] / op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -900,6 +1262,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = op[0]->value.d[c0] - op[1]->value.d[c1] * floor(op[0]->value.d[c0] / op[1]->value.d[c1]);
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[1]->value.u64[c1] == 0 ? 0 : op[0]->value.u64[c0] % op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[1]->value.i64[c1] == 0 ? 0 : op[0]->value.i64[c0] % op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -921,6 +1289,12 @@
          case GLSL_TYPE_DOUBLE:
             data.b[c] = op[0]->value.d[c] < op[1]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.b[c] = op[0]->value.u64[c] < op[1]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] < op[1]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -942,6 +1316,12 @@
          case GLSL_TYPE_DOUBLE:
             data.b[c] = op[0]->value.d[c] > op[1]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.b[c] = op[0]->value.u64[c] > op[1]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] > op[1]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -963,6 +1343,12 @@
          case GLSL_TYPE_DOUBLE:
             data.b[c] = op[0]->value.d[c] <= op[1]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.b[c] = op[0]->value.u64[c] <= op[1]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] <= op[1]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -984,6 +1370,12 @@
          case GLSL_TYPE_DOUBLE:
             data.b[c] = op[0]->value.d[c] >= op[1]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.b[c] = op[0]->value.u64[c] >= op[1]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] >= op[1]->value.i64[c];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1005,6 +1397,12 @@
          case GLSL_TYPE_DOUBLE:
             data.b[c] = op[0]->value.d[c] == op[1]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.b[c] = op[0]->value.u64[c] == op[1]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] == op[1]->value.i64[c];
+            break;
          case GLSL_TYPE_BOOL:
             data.b[c] = op[0]->value.b[c] == op[1]->value.b[c];
             break;
@@ -1029,6 +1427,12 @@
          case GLSL_TYPE_DOUBLE:
             data.b[c] = op[0]->value.d[c] != op[1]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.b[c] = op[0]->value.u64[c] != op[1]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.b[c] = op[0]->value.i64[c] != op[1]->value.i64[c];
+            break;
          case GLSL_TYPE_BOOL:
             data.b[c] = op[0]->value.b[c] != op[1]->value.b[c];
             break;
@@ -1048,9 +1452,13 @@
 
    case ir_binop_lshift:
       assert(op[0]->type->base_type == GLSL_TYPE_UINT ||
-             op[0]->type->base_type == GLSL_TYPE_INT);
+             op[0]->type->base_type == GLSL_TYPE_INT ||
+             op[0]->type->base_type == GLSL_TYPE_UINT64 ||
+             op[0]->type->base_type == GLSL_TYPE_INT64);
       assert(op[1]->type->base_type == GLSL_TYPE_UINT ||
-             op[1]->type->base_type == GLSL_TYPE_INT);
+             op[1]->type->base_type == GLSL_TYPE_INT ||
+             op[1]->type->base_type == GLSL_TYPE_UINT64 ||
+             op[1]->type->base_type == GLSL_TYPE_INT64);
       for (unsigned c = 0, c0 = 0, c1 = 0;
            c < components;
            c0 += c0_inc, c1 += c1_inc, c++) {
@@ -1062,6 +1470,12 @@
          case GLSL_TYPE_INT:
             data.i[c] = op[0]->value.i[c0] << op[1]->value.i[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] << op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] << op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1070,9 +1484,13 @@
 
    case ir_binop_rshift:
       assert(op[0]->type->base_type == GLSL_TYPE_UINT ||
-             op[0]->type->base_type == GLSL_TYPE_INT);
+             op[0]->type->base_type == GLSL_TYPE_INT ||
+             op[0]->type->base_type == GLSL_TYPE_UINT64 ||
+             op[0]->type->base_type == GLSL_TYPE_INT64);
       assert(op[1]->type->base_type == GLSL_TYPE_UINT ||
-             op[1]->type->base_type == GLSL_TYPE_INT);
+             op[1]->type->base_type == GLSL_TYPE_INT ||
+             op[1]->type->base_type == GLSL_TYPE_UINT64 ||
+             op[1]->type->base_type == GLSL_TYPE_INT64);
       for (unsigned c = 0, c0 = 0, c1 = 0;
            c < components;
            c0 += c0_inc, c1 += c1_inc, c++) {
@@ -1084,6 +1502,12 @@
          case GLSL_TYPE_INT:
             data.i[c] = op[0]->value.i[c0] >> op[1]->value.i[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] >> op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] >> op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1103,6 +1527,12 @@
          case GLSL_TYPE_INT:
             data.i[c] = op[0]->value.i[c0] & op[1]->value.i[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] & op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] & op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1122,6 +1552,12 @@
          case GLSL_TYPE_INT:
             data.i[c] = op[0]->value.i[c0] ^ op[1]->value.i[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] ^ op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] ^ op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1141,6 +1577,12 @@
          case GLSL_TYPE_INT:
             data.i[c] = op[0]->value.i[c0] | op[1]->value.i[c1];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.u64[c0] | op[1]->value.u64[c1];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.i64[c0] | op[1]->value.i64[c1];
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1215,6 +1657,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = MIN2(op[0]->value.d[c0], op[1]->value.d[c1]);
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = MIN2(op[0]->value.u64[c0], op[1]->value.u64[c1]);
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = MIN2(op[0]->value.i64[c0], op[1]->value.i64[c1]);
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1240,6 +1688,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = MAX2(op[0]->value.d[c0], op[1]->value.d[c1]);
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = MAX2(op[0]->value.u64[c0], op[1]->value.u64[c1]);
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = MAX2(op[0]->value.i64[c0], op[1]->value.i64[c1]);
+            break;
          default:
             unreachable("invalid type");
          }
@@ -1290,6 +1744,12 @@
       case GLSL_TYPE_DOUBLE:
          data.d[0] = op[0]->value.d[c];
          break;
+      case GLSL_TYPE_UINT64:
+         data.u64[0] = op[0]->value.u64[c];
+         break;
+      case GLSL_TYPE_INT64:
+         data.i64[0] = op[0]->value.i64[c];
+         break;
       case GLSL_TYPE_BOOL:
          data.b[0] = op[0]->value.b[c];
          break;
@@ -1315,12 +1775,9 @@
       break;
 
    case ir_triop_lrp: {
-      assert(op[0]->type->base_type == GLSL_TYPE_FLOAT ||
-             op[0]->type->base_type == GLSL_TYPE_DOUBLE);
-      assert(op[1]->type->base_type == GLSL_TYPE_FLOAT ||
-             op[1]->type->base_type == GLSL_TYPE_DOUBLE);
-      assert(op[2]->type->base_type == GLSL_TYPE_FLOAT ||
-             op[2]->type->base_type == GLSL_TYPE_DOUBLE);
+      assert(op[0]->type->is_float() || op[0]->type->is_double());
+      assert(op[1]->type->is_float() || op[1]->type->is_double());
+      assert(op[2]->type->is_float() || op[2]->type->is_double());
 
       unsigned c2_inc = op[2]->type->is_scalar() ? 0 : 1;
       for (unsigned c = 0, c2 = 0; c < components; c2 += c2_inc, c++) {
@@ -1353,6 +1810,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = op[0]->value.b[c] ? op[1]->value.d[c] : op[2]->value.d[c];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[0]->value.b[c] ? op[1]->value.u64[c] : op[2]->value.u64[c];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[0]->value.b[c] ? op[1]->value.i64[c] : op[2]->value.i64[c];
+            break;
          case GLSL_TYPE_BOOL:
             data.b[c] = op[0]->value.b[c] ? op[1]->value.b[c] : op[2]->value.b[c];
             break;
@@ -1395,6 +1858,12 @@
       case GLSL_TYPE_DOUBLE:
          data.d[idx] = op[1]->value.d[0];
          break;
+      case GLSL_TYPE_UINT64:
+         data.u64[idx] = op[1]->value.u64[0];
+         break;
+      case GLSL_TYPE_INT64:
+         data.i64[idx] = op[1]->value.i64[0];
+         break;
       case GLSL_TYPE_BOOL:
          data.b[idx] = op[1]->value.b[0];
          break;
@@ -1434,6 +1903,12 @@
          case GLSL_TYPE_DOUBLE:
             data.d[c] = op[c]->value.d[0];
             break;
+         case GLSL_TYPE_UINT64:
+            data.u64[c] = op[c]->value.u64[0];
+            break;
+         case GLSL_TYPE_INT64:
+            data.i64[c] = op[c]->value.i64[0];
+            break;
          case GLSL_TYPE_BOOL:
             data.b[c] = op[c]->value.b[0];
             break;
